#include <stdio.h>
#include <stdlib.h>
#include <string.h>

char *prog;

#define P printf

void
usage() {
    P("Use:\n\t%s [n] command...\n", prog);
    P("\n");
    P("%s copies the first n (default 1) lines "
		"from stdin to stdout, then execs\n", prog);
    P("<command> to filter the rest of the input.\n");
    P("It is useful for copying a header to stdout before filtering the\n");
    P("rest of the data, since typical filters discard the header.\n");
    P("Note that %s is only useful if <command> filters stdin!\n", prog);
    P("\n");
    P("Example:\n");
    P("\t$ ps -ef | %s grep bash\n", prog);
    P("\tUID        PID  PPID  C STIME TTY          TIME CMD\n");
    P("\twill     20306  6535  0 13:14 pts/9    00:00:00 hdr grep bash\n");

}

int
main(int argc, char **argv) {
    char *s;
    int i;
    unsigned char c;
    unsigned long n;

    setlinebuf(stdin);

    prog = strrchr(argv[0], '/');
    prog = prog ? prog+1 : argv[0];

    if (argc == 1 || argv[1][0] == '-' || argv[1][0] == '\0') {
	usage();
	exit(1);
    }
    n = strtoul(argv[1], &s, 10);
    if (*s == '\0') {
	argv++;
	argc--;
	if (argc == 1) {
	    usage();
	    exit(1);
	}
    } else {
	n = 1;
    }
    while (n) {
	if ((i=read(0, &c, 1)) <= 0) {
	    if (i < 0) {
		fprintf(stderr, "%s: failed to read stdin: ", prog);
		perror("");
		exit(1);
	    }
	    exit(0);
	}
	if (c == '\n')
	    n--;
	putchar((int) c);
    }
    if (execvp(argv[1], &argv[1]) == -1) {
	fprintf(stderr, "%s: failed to exec \"%s\": ", prog, argv[1]);
	perror("");
	exit(1);
    }
}
